#!/usr/bin/env python
# -*- coding: utf-8 -*-

import gobject
from zeitgeist.datamodel import Event, Subject, Interpretation, Manifestation
from gi.repository import GtkChamplain
from gi.repository import Champlain
from datasink import SUPPORTED_SOURCES, datasink, match_interpretation
from gi.repository import Gtk
from ui_utils import icon_factory, no_pixbuf, get_icon

from markers import create_marker_layer

class LauncherGTK:
    def __init__(self):
        self.window = Gtk.Window()
        self.window.set_border_width(10)
        self.window.set_title("El Loco")
        self.window.connect("destroy", Gtk.main_quit)
        
        vbox = Gtk.VBox(False, 12)
        hbox = Gtk.HBox()
        
        self.sidebar = Sidebar()
        hbox.pack_end(self.sidebar, False, False)

        embed = GtkChamplain.ChamplainEmbed()
        embed.set_size_request(640, 480)

        self.view = embed.get_view()
        self.view.set_reactive(True)
        self.view.connect('button-press-event', self.mouse_click_cb, self.view)

        self.view.set_property('scroll-mode', champlain.SCROLL_MODE_KINETIC)
        self.view.set_property('zoom-level', 5)
        #self.view.set_property('license-text', "Don't eat cereals with orange juice\nIt tastes bad")
        self.view.set_property('show-scale', True)
        
        self.radius =  1.0/5
        
        self.latitude = -1
        self.longitude = -1
        

        self.layer = create_marker_layer(self.view)
        self.view.add_layer(self.layer)
        self.layer.hide_all_markers()
        
        
        self.polygon = polygon = champlain.Polygon()
        # Cheap approx of Highway 10
       
        polygon.set_stroke_width(5.0);
        polygon.set_property("mark-points", True)
        self.view.add_polygon(polygon)

        bbox = Gtk.HBox(False, 6)
        button = Gtk.Button(stock=Gtk.STOCK_ZOOM_IN)
        button.connect("clicked", self.zoom_in)
        bbox.add(button)

        button = Gtk.Button(stock=Gtk.STOCK_ZOOM_OUT)
        button.connect("clicked", self.zoom_out)
        bbox.add(button)

        button = Gtk.ToggleButton(label="Markers")
        button.set_active(False)
        button.connect("toggled", self.toggle_layer)
        #bbox.add(button)

        combo = Gtk.combo_box_new_text()
        self.map_source_factory = champlain.map_source_factory_dup_default()
        liststore = Gtk.ListStore(str, str)
        for source in self.map_source_factory.dup_list():
            liststore.append([source.id, source.name])
            combo.append_text(source.name)
        combo.set_model(liststore)
        combo.set_attributes(combo.get_cells()[0], text=1)
        combo.set_active(0)
        combo.connect("changed", self.map_source_changed)
        bbox.add(combo)

        self.spinbutton = Gtk.SpinButton(Gtk.Adjustment(lower=0, upper=20, 
            value=1, step_incr=1))
        self.spinbutton.connect("changed", self.zoom_changed)
        self.view.connect("notify::zoom-level", self.map_zoom_changed)
        self.spinbutton.set_value(5)
        bbox.add(self.spinbutton)

        button = Gtk.Image()
        self.view.connect("notify::state", self.view_state_changed, button)
        bbox.pack_end(button, False, False, 0)        

        vbox.pack_start(bbox, expand=False, fill=False)
        vbox.add(embed)

        hbox.pack_start(vbox)
        self.window.add(hbox)
        datasink.connect("reload", self.reload)


        self.window.show_all()

    def reload(self, widget):
        zoom = self.view.get_zoom_level()
        self.sidebar.set_location((self.longitude, self.latitude, 2.0/zoom))


    def zoom_in(self, widget):
        self.view.zoom_in()

    def zoom_out(self, widget):
        self.view.zoom_out()

    def toggle_layer(self, widget):
        if widget.get_active():
            self.polygon.show()    
            self.layer.animate_in_all_markers()
        else:
            self.polygon.hide()    
            self.layer.animate_out_all_markers()

    def mouse_click_cb(self, actor, event, view):
        lat, lon = view.get_coords_from_event(event)
        print "Mouse click at: %f %f" % (lon, lat)
        self.longitude = lon
        self.latitude = lat
        
        self.polygon.clear_points()
        self.polygon.append_point(lat, lon)
        self.polygon.show_all()
        
        zoom = self.view.get_zoom_level()
        self.sidebar.set_location((self.longitude, self.latitude, 2.0/zoom))
        
        
    def zoom_changed(self, widget):
        self.view.set_property("zoom-level", self.spinbutton.get_value_as_int())
        
        if self.latitude and self.longitude:
            zoom = self.view.get_zoom_level()
            self.sidebar.set_location((self.longitude, self.latitude, 2.0/zoom))

    def map_source_changed(self, widget):
        model = widget.get_model()
        iter = widget.get_active_iter()
        id = model.get_value(iter, 0)
        source = self.map_source_factory.create(id);
        self.view.set_property("map-source", source)

    def map_zoom_changed(self, widget, value):
        self.spinbutton.set_value(self.view.get_property("zoom-level"))

    def view_state_changed(self, view, paramspec, image):
        state = view.get_property("state")
        if state == champlain.STATE_LOADING:
            image.set_from_stock(Gtk.STOCK_NETWORK, Gtk.ICON_SIZE_BUTTON)
        else:
            image.clear()
            


class Sidebar(Gtk.HBox):
    def __init__(self):
        Gtk.HBox.__init__(self)
        self.label = Gtk.Label("Location Activities")
        self.location = (-1, -1, 1)
        self.iconview = TreeView()
        self.notebook = Gtk.Notebook()
        self.set_size_request(400,-1)
        vbox = Gtk.VBox()
        vbox.pack_start(Gtk.HSeparator(), False, False)
        vbox.pack_start(self.iconview)
        self.notebook.append_page(vbox, self.label)
        self.pack_start(Gtk.Label(), False, False, 3)
        self.pack_start(self.notebook, True, True)
        self.notebook.set_tab_label_packing(vbox, True, True, Gtk.PACK_START)
        
    def set_location(self, location):
        self.location = location
        events = datasink.get_events(location)
        self.iconview.set_categories(events)
          
          
class TreeView(Gtk.TreeView):
    def __init__(self):
        Gtk.TreeView.__init__(self)
        self.model = Gtk.TreeStore(Gtk.gdk.Pixbuf, str, gobject.TYPE_PYOBJECT)
        
        self.icon_cell = Gtk.CellRendererPixbuf()
        self.icon_cell.set_property("xalign", 0.5)
        self.icon_cell.set_property("yalign", 0.5)
        icon_column = Gtk.TreeViewColumn()
        icon_column.pack_start(self.icon_cell)
        icon_column.add_attribute(self.icon_cell, "pixbuf", 0)
        
        self.text_cell = Gtk.CellRendererText()
        self.text_cell.set_property("yalign", 0.5)
        self.text_cell.set_property("xalign", 0.0)
        self.text_cell.set_property("ellipsize", pango.ELLIPSIZE_END)
        text_column = Gtk.TreeViewColumn()
        text_column.pack_start(self.text_cell, expand=True)
        #self.text_cell.set_fixed_size(128, -1)
        text_column.add_attribute(self.text_cell, "markup", 1)
                
        self.append_column(icon_column)
        self.append_column(text_column)
                
        self.use_cells = isinstance(self, Gtk.CellLayout)
        self.set_double_buffered(True)
        self.set_reallocate_redraws(True)
        self.set_resize_mode(Gtk.RESIZE_IMMEDIATE)
        self.set_model(self.model)
        #self.set_rules_hint(True)
        self.set_grid_lines(Gtk.TREE_VIEW_GRID_LINES_NONE)
        self.set_headers_visible(False)
        self.connect("row-activated", self.launch)
        self.items = []
        self.categories = {}
        #self.set_categories()
        self.show_all()
        
    def set_categories(self, events):
        self.clear()
        if events:
            category = events.keys()
            category.sort()
            
            for key in category:
                icon = icon_factory.load_icon(SUPPORTED_SOURCES[key]["icon"], 16)
                if len(events[key]) > 0:
                    self.categories[key] = self.model.append(None, [icon, "<b>"+SUPPORTED_SOURCES[key]["name"]+"</b>", None])
                    for event in events[key]:
                        self.add_item(event, 16)
        self.expand_all()
        
        
    def launch(self, view, path, w):
        item = self.model[path][2]
        launch.launch_uri(item.uri)
        
    def clear(self):
        self.model.clear()
        
        
    
    def add_item(self, item, icon_size):
        item = item.subjects[0]           

        iter = self.model.append(self.categories[item.interpretation],[no_pixbuf, item.text, item])
        def callback(icon):
            self.model[iter][0] = icon if icon is not None else no_pixbuf
        if not item.interpretation == Interpretation.IMMESSAGE.uri:
            get_icon(item, callback)
        else:
            self.model[iter][0] = icon_factory.load_icon("stock_person", 16, True)
        
          
if __name__ == "__main__":
    gobject.threads_init()
    LauncherGTK()
    Gtk.main()
